@@ -1,29 +1,47 @@
 from __future__ import with_statement
-import unittest, doctest
-import os, shutil, os.path
-import hgapi 
 
+import unittest
+import doctest
+
+import os
+
+import shutil
+
+import hgapi
+
+
+# TODO: add better logger test
 class TestHgAPI(unittest.TestCase):
-    """Tests for hgapi.py
-    Uses and wipes subfolder named 'test'
-    Tests are dependant on each other; named test_<number>_name for sorting
+    """
+        Test the hgapi.
+
+        Uses and wipes folders named 'test' (a.k.a repo), 'test-clone'
+        (a.k.a clone).  Tests are dependent on each other; named
+        test_<number>_name for sorting.
     """
     repo = hgapi.Repo("./test", user="testuser")
-    
+    clone = hgapi.Repo("./test-clone", user="testuser")
+
+    @classmethod
+    def _delete_and_create(cls, path):
+        if os.path.exists(path):
+            shutil.rmtree(path)
+        os.mkdir(path)
+        assert os.path.exists(path)
+
     @classmethod
     def setUpClass(cls):
-        #Patch Python 3
+        # patch for Python 3
         if hasattr(cls, "assertEqual"):
             setattr(cls, "assertEquals", cls.assertEqual)
             setattr(cls, "assertNotEquals", cls.assertNotEqual)
-        if os.path.exists("./test"):
-            shutil.rmtree("./test")
-        os.mkdir("./test")
-        assert os.path.exists("./test")
+        TestHgAPI._delete_and_create("./test")
+        TestHgAPI._delete_and_create("./original")
 
     @classmethod
     def tearDownClass(self):
-        shutil.rmtree("test")
+        shutil.rmtree("test", ignore_errors=True)
+        shutil.rmtree("test-clone", ignore_errors=True)
 
     def test_000_Init(self):
         self.repo.hg_init()
@@ -39,22 +57,32 @@ class TestHgAPI(unittest.TestCase):
         with open("test/file.txt", "w") as out:
             out.write("stuff")
         self.repo.hg_add("file.txt")
-        
+        self.assertListEqual(self.repo.hg_status()['A'], ['file.txt'])
+
+    def test_021_Add(self):
+        with open("test/foo.txt", "w") as out:
+            out.write("A sample file")
+        with open("test/bar.txt", "w") as out:
+            out.write("Another sample file")
+        self.repo.hg_add()
+        self.assertListEqual(self.repo.hg_status()['A'],
+                             ['bar.txt', 'file.txt', 'foo.txt'])
+
     def test_030_Commit(self):
-        #Commit and check that we're on a real revision
+        # commit and check that we're on a real revision
         self.repo.hg_commit("adding", user="test")
-        rev  = self.repo.hg_rev()
+        rev = self.repo.hg_rev()
         hgid = self.repo.hg_id()
         self.assertEquals(rev, 0)
         self.assertNotEquals(hgid, "000000000000")
 
-        #write some more to file
+        # write some more to file
         with open("test/file.txt", "w+") as out:
             out.write("more stuff")
 
-        #Commit and check that changes have been made
+        # commit and check that changes have been made
         self.repo.hg_commit("modifying", user="test")
-        rev2  = self.repo.hg_rev()
+        rev2 = self.repo.hg_rev()
         hgid2 = self.repo.hg_id()
         self.assertNotEquals(rev, rev2)
         self.assertNotEquals(hgid, hgid2)
@@ -73,7 +101,6 @@ class TestHgAPI(unittest.TestCase):
         self.repo.hg_update("tip")
         self.assertEquals(self.repo.hg_id(), node)
 
-
     def test_060_Heads(self):
         node = self.repo.hg_node()
 
@@ -81,7 +108,7 @@ class TestHgAPI(unittest.TestCase):
         with open("test/file.txt", "w+") as out:
             out.write("even more stuff")
 
-        #creates new head
+        # creates new head
         self.repo.hg_commit("modifying", user="test")
 
         heads = self.repo.hg_heads()
@@ -94,11 +121,11 @@ class TestHgAPI(unittest.TestCase):
         self.assertTrue(node[:12] in heads)
         self.assertTrue(self.repo.hg_node()[:12] in heads)
 
-        #Close head again
+        # close head again
         self.repo.hg_commit("Closing branch", close_branch=True)
         self.repo.hg_update(node)
 
-        #Check that there's only one head remaining
+        # check that there's only one head remaining
         heads = self.repo.hg_heads()
         self.assertEquals(len(heads), 1)
         self.assertTrue(node in heads)
@@ -112,84 +139,84 @@ class TestHgAPI(unittest.TestCase):
                        "stuff.list = one two three\n" +
                        "[ui]\n" +
                        "username = testsson")
-        #re-read config
-        self.repo.read_config()     
+        # re-read config
+        self.repo.read_config()
         self.assertEquals(self.repo.config('test', 'stuff.otherstuff'),
                           "tsosvalue")
         self.assertEquals(self.repo.config('ui', 'username'),
                           "testsson")
 
-
     def test_071_ConfigBool(self):
         self.assertTrue(self.repo.configbool('test', 'stuff.debug'))
         self.assertFalse(self.repo.configbool('test', 'stuff.verbose'))
-        
+
     def test_072_ConfigList(self):
         self.assertTrue(self.repo.configlist('test', 'stuff.list'),
                         ["one", "two", "three"])
 
     def test_080_LogBreakage(self):
-        """Some log messages/users could possibly break 
-        the revision parsing"""
-        #write some more to file
+        """
+            Some log messages/users could possibly break
+            the revision parsing.
+        """
+        # write some more to file
         with open("test/file.txt", "w+") as out:
             out.write("stuff and, more stuff")
 
-        #Commit and check that changes have been made
+        # commit and check that changes have been made
         self.repo.hg_commit("}", user="},desc=\"test")
         self.assertEquals(self.repo["tip"].desc, "}")
         self.assertEquals(self.repo["tip"].author, "},desc=\"test")
-  
 
     def test_090_ModifiedStatus(self):
-        #write some more to file
+        # write some more to file
         with open("test/file.txt", "a") as out:
             out.write("stuff stuff stuff")
         status = self.repo.hg_status()
-        self.assertEquals(status, 
-                          {'A': [], 'M': ['file.txt'], '!': [], 
+        self.assertEquals(status,
+                          {'A': [], 'M': ['file.txt'], '!': [],
                            '?': [], 'R': []})
-        
+
     def test_100_CleanStatus(self):
-        #commit file created in 090
+        # commit file created in 090
         self.repo.hg_commit("Comitting changes", user="test")
-        #Assert status is empty
-        self.assertEquals(self.repo.hg_status(), 
+        # assert status is empty
+        self.assertEquals(self.repo.hg_status(),
                           {'A': [], 'M': [], '!': [], '?': [], 'R': []})
 
     def test_110_UntrackedStatus(self):
-        #Create a new file
+        # create a new file
         with open("test/file2.txt", "w") as out:
             out.write("stuff stuff stuff")
         status = self.repo.hg_status()
-        self.assertEquals(status, 
-                          {'A': [], 'M': [], '!': [], 
+        self.assertEquals(status,
+                          {'A': [], 'M': [], '!': [],
                            '?': ['file2.txt'], 'R': []})
 
     def test_120_AddedStatus(self):
-        #Add file created in 110
+        # add file created in 110
         self.repo.hg_add("file2.txt")
         status = self.repo.hg_status()
-        self.assertEquals(status, 
-                          {'A': ['file2.txt'], 'M': [], '!': [], 
+        self.assertEquals(status,
+                          {'A': ['file2.txt'], 'M': [], '!': [],
                            '?': [], 'R': []})
 
     def test_130_MissingStatus(self):
-        #Commit file created in 120
+        # commit file created in 120
         self.repo.hg_commit("Added file")
         import os
         os.unlink("test/file2.txt")
         status = self.repo.hg_status()
-        self.assertEquals(status, 
-                          {'A': [], 'M': [], '!': ['file2.txt'], 
+        self.assertEquals(status,
+                          {'A': [], 'M': [], '!': ['file2.txt'],
                            '?': [], 'R': []})
 
     def test_140_RemovedStatus(self):
-        #Remove file from repo
+        # remove file from repo
         self.repo.hg_remove("file2.txt")
         status = self.repo.hg_status()
-        self.assertEquals(status, 
-                          {'A': [], 'M': [], '!': [], 
+        self.assertEquals(status,
+                          {'A': [], 'M': [], '!': [],
                            '?': [], 'R': ['file2.txt']})
 
     def test_140_EmptyStatus(self):
@@ -198,14 +225,14 @@ class TestHgAPI(unittest.TestCase):
         self.assertEquals(status, {})
 
     def test_150_ForkAndMerge(self):
-        #Store this version
+        # store this version
         node = self.repo.hg_node()
 
         self.repo.hg_update(4, clean=True)
         with open("test/file3.txt", "w") as out:
             out.write("this is more stuff")
 
-        #creates new head
+        # creates new head
         self.repo.hg_add("file3.txt")
         self.repo.hg_commit("adding head", user="test")
 
@@ -214,40 +241,42 @@ class TestHgAPI(unittest.TestCase):
         self.assertTrue(node in heads)
         self.assertTrue(self.repo.hg_node() in heads)
 
-        #merge the changes
+        # merge the changes
         self.repo.hg_merge(node)
         self.repo.hg_commit("merge")
-        
-        #Check that there's only one head remaining
+
+        # check that there's only one head remaining
         heads = self.repo.hg_heads()
         self.assertEquals(len(heads), 1)
 
     def test_160_CommitFiles(self):
         with open("test/file2.txt", "w") as out:
-                    out.write("newstuff")        	
+                    out.write("newstuff")
         with open("test/file3.txt", "w") as out:
             out.write("this is even more stuff")
-        self.repo.hg_commit("only committing file2.txt", user="test", files=["file2.txt"])
+        self.repo.hg_commit("only committing file2.txt",
+                            user="test",
+                            files=["file2.txt"])
         self.assertTrue("file3.txt" in self.repo.hg_status()["M"])
-        
+
     def test_170_Indexing(self):
         with open("test/file2.txt", "a+") as out:
             out.write("newstuff")
         self.repo.hg_commit("indexing", user="test", files=["file2.txt"])
-        #Compare tip and current revision number
+        # compare tip and current revision number
         self.assertEquals(self.repo['tip'], self.repo[self.repo.hg_rev()])
         self.assertEquals(self.repo['tip'].desc, "indexing")
-        
+
     def test_180_Slicing(self):
         with open("test/file2.txt", "a+") as out:
             out.write("newstuff")
         self.repo.hg_commit("indexing", user="test", files=["file2.txt"])
-        
+
         all_revs = self.repo[0:'tip']
         self.assertEquals(len(all_revs), 12)
         self.assertEquals(all_revs[-1].desc, all_revs[-2].desc)
         self.assertNotEquals(all_revs[-2].desc, all_revs[-3].desc)
-        
+
     def test_190_Branches(self):
         # make sure there is only one branch and it is default
         self.assertEquals(self.repo.hg_branch(), "default")
@@ -257,8 +286,8 @@ class TestHgAPI(unittest.TestCase):
         self.assertEquals(len(branch_names), 1)
         self.assertEquals(branch_names[0], "default")
 
-        # create a new branch, should still be default in branches until we commit
-        # but branch should return the new branch
+        # create a new branch, should still be default in branches
+        # until we commit - but branch should return the new branch
         self.assertTrue(self.repo.hg_branch('test_branch').startswith(
             "marked working directory as branch test_branch"))
         self.assertEquals(self.repo.hg_branch(), "test_branch")
@@ -282,7 +311,9 @@ class TestHgAPI(unittest.TestCase):
         with open("test/file.txt", "w+") as out:
             out.write("even more stuff")
 
-        self.repo.hg_commit("modifying and setting a date", user="test", date="10/10/11 UTC")
+        self.repo.hg_commit("modifying and setting a date",
+                            user="test",
+                            date="10/10/11 UTC")
 
         rev = self.repo["tip"]
         self.assertEquals(rev.desc, "modifying and setting a date")
@@ -298,15 +329,18 @@ class TestHgAPI(unittest.TestCase):
         self.repo.hg_tag('long mytag3', rev=2)
         tags = self.repo.hg_tags()
         self.assertEqual(tags, {'mytag': original_tip,
-            'othertag': original_tip,
-            'mytag2': self.repo[1].node,
-            'long mytag3': self.repo[2].node,
-            'tip': self.repo[-1].node})
+                                'othertag': original_tip,
+                                'mytag2': self.repo[1].node,
+                                'long mytag3': self.repo[2].node,
+                                'tip': self.repo[-1].node})
 
     def test_220_LogWithBranch(self):
         default = self.repo.hg_log(branch='default')
         branch = self.repo.hg_log(branch='test_branch')
 
+        print(default)
+        print(branch)
+
         self.assertTrue("commit test_branch" in branch)
         self.assertFalse("commit test_branch" in default)
 
@@ -316,11 +350,13 @@ class TestHgAPI(unittest.TestCase):
         self.assertTrue('+even more stuff' in diffs[1]['diff'])
 
     def test_240_DiffFile(self):
-        diffs = self.repo.hg_diff('default', 'test_branch', filenames=['file.txt'])
+        diffs = self.repo.hg_diff('default',
+                                  'test_branch',
+                                  filenames=['file.txt'])
         self.assertEquals(len(diffs), 1)
         self.assertEquals(diffs[0]['filename'], 'file.txt')
         self.assertTrue('+even more stuff' in diffs[0]['diff'])
-        
+
     def test_250_ExitCode(self):
         try:
             self.repo.hg_update('notexistingref')
@@ -333,29 +369,94 @@ class TestHgAPI(unittest.TestCase):
         diffs = self.repo.hg_diff('default', filenames=['file.txt'])
         self.assertEquals(len(diffs), 0)
 
+    def test_270_Move(self):
+        # add source.txt, commit it
+        with open("test/source.txt", "w") as out:
+            out.write("stuff")
+        self.repo.hg_add("source.txt")
+        self.repo.hg_commit("Source is committed.")
+        # move it to destination
+        self.repo.hg_rename("source.txt", "destination.txt")
+        # get diffs and check proper move
+        diffs = self.repo.hg_diff()
+        self.assertTrue(diffs[0]['filename'] == 'destination.txt')
+        self.assertTrue(diffs[1]['filename'] == 'source.txt')
+        self.repo.hg_commit("Checked move.")
+
+    def test_280_AddRemove(self):
+        # remove foo and add fizz first
+        os.remove("test/foo.txt")
+        with open("test/fizz.txt", "w") as out:
+            out.write("fuzz")
+        # then test addremove
+        self.repo.hg_addremove()
+        self.assertListEqual(self.repo.hg_status()['A'], ['fizz.txt'])
+        self.assertListEqual(self.repo.hg_status()['R'], ['foo.txt'])
+
+    def test_300_clone(self):
+        # clone test to test clone
+        self.clone = hgapi.Repo.hg_clone("./test", "./test-clone")
+        self.assertTrue(isinstance(self.clone, hgapi.Repo))
+        self.assertEquals(self.clone.path, self.repo.path + "-clone")
+
+    def test_310_pull(self):
+        # add a new directory with some files in test repo first
+        os.mkdir("./test/cities")
+        with open("./test/cities/brussels.txt", "w") as out:
+            out.write("brussel")
+        with open("./test/cities/antwerp.txt", "w") as out:
+            out.write("antwerpen")
+        self.repo.hg_add()
+        message = "[TEST] Added two cities."
+        self.repo.hg_commit(message)
+        self.clone.hg_pull("../test")
+        # update clone after pull and then check if the
+        # identifiers are the same
+        self.clone.hg_update("tip")
+        self.assertEquals(self.clone.hg_id(), self.repo.hg_id())
+        # check summary of pulled tip
+        self.assertTrue(message in self.clone.hg_log(identifier="tip"))
+
+    def test_320_push(self):
+        # add another file in test-clone first
+        with open("./test-clone/cities/ghent.txt", "w") as out:
+            out.write("gent")
+        self.clone.hg_add()
+        message = "[CLONE] Added one file."
+        self.clone.hg_commit(message)
+        self.clone.hg_push("../test")
+        # update test after push and assert
+        self.repo.hg_update("tip")
+        self.assertEquals(self.clone.hg_id(), self.repo.hg_id())
+        # check summary of pushed tip
+        self.assertTrue(message in self.repo.hg_log(identifier="tip"))
+
     def test_400_version(self):
         self.assertNotEquals(hgapi.Repo.hg_version(), "")
 
-    def test_410_clone(self):
-        repo = hgapi.Repo.hg_clone("./test", "./test-clone")
-        self.assertTrue(isinstance(repo, hgapi.Repo))
-        self.assertEquals(repo.path, self.repo.path + "-clone")
-        shutil.rmtree("test-clone")
+    def test_410_root(self):
+        # regular test repo
+        reply = hgapi.Repo.hg_root("./test")
+        self.assertTrue(reply.endswith("/hgapi/test"))
+        # non existing repo
+        self.assertRaises(hgapi.HgException, hgapi.Repo.hg_root, "./whatever")
+
 
 def test_doc():
-    #Prepare for doctest
+    # prepare for doctest
     os.mkdir("./test_hgapi")
     with open("test_hgapi/file.txt", "w") as target:
         w = target.write("stuff")
     try:
-        #Run doctest
+        # run doctest
         res = doctest.testfile("../README.rst")
     finally:
-        #Cleanup
+        # cleanup
         shutil.rmtree("test_hgapi")
 
+
 if __name__ == "__main__":
-    import sys
+    # run full test suite
     try:
         test_doc()
     finally:
